from pandac.PandaModules import *
from direct.showbase.DirectObject import DirectObject
from direct.gui.OnscreenText import OnscreenText
from time import time
from traceback import print_exc
from systems import *
from buildings import *
from maps import *
from resources import *
from direct.gui.DirectGui import DirectButton
from progression import progression


class Game(object,DirectObject):
    def __init__(self):
        import __builtin__
        __builtin__.game = self
        load_island()
        
        
        
        self.moveVec = Vec3(0,0,0)
        taskMgr.add(self.mouse, 'mouse')
        taskMgr.add(self.update, 'update')
        self.accept( "w" , self.up )
        self.accept( "s" , self.down )
        self.accept( "space" , self.get_point )
        self.init_lights()
        self.init_picking()
        self.resources = Resources()
        self.init_lines()
        
        
        self.buttons = progression[0][1]
        self.init_gui()
        
        self.bs = BuildingSystem()
        self.rs = RoadSystem()
        self.ws = WaterSystem()
        self.ps = PowerSystem()
        self.ds = DemandSystem()
        self.ms = MessageSystem()
        self.rv = RevenueSystem()
        
        self.selected = None
        
        self.ring = loader.loadModel('data/ring.egg')
        self.ring.reparentTo(render)
        
    def init_gui(self):
        self.info = OnscreenText("",
        style=1, fg=(1,1,1,1), pos=(-1.3, 0.95),
        align=TextNode.ALeft, scale = .05, mayChange= True)
        self.out = OnscreenText("messages...",
        style=1, fg=(1,1,1,1), pos=(-1.3, -0.80),
        align=TextNode.ALeft, scale = .05, mayChange= True)
        
        self.help_text = OnscreenText("messages...\n"*10,
        style=1, fg=(1,1,1,1), pos=(.4, -.4),
        align=TextNode.ALeft, scale = .05, mayChange= True)
        
        self.rotating = False
        self.zooming = False
        self.heading = 0
        self.pitch = -45
        base.accept("mouse2", self.__setattr__,['rotating',True])
        base.accept("mouse2-up", self.__setattr__,['rotating',False])
        
        
        
        base.accept("wheel_up", self.__setattr__,['zooming',-1])
        base.accept("wheel_down", self.__setattr__,['zooming',1])
        base.accept("mouse3",self.jump_cam)
        base.accept("mouse3-up", self.__setattr__,['rotating',False])
        
        base.zoomCam = NodePath('zoomCam')
        base.zoomCam.reparentTo(render)
        base.camera.reparentTo(base.zoomCam)
        
       
        self.build_tree = progression[0][1]
        self.button_node = NodePath("buttonNode")
        self.button_node.reparentTo(aspect2d)
        for y,branch in enumerate(self.build_tree):
            for x,building_type in enumerate(branch):
                self.make_button(building_type,len(branch)-x,y)
         
        base.accept("c", self.build_here,[Command])
        base.accept("p", self.build_here,[Residential])
        base.accept("f", self.build_here,[Farm])
        base.accept("a", self.build_here,[Aqua])
      
        base.accept("r", self.build_here,[RoadNode])
        base.accept("t", self.build_here,[WaterNode])
        base.accept("y", self.build_here,[EnergyNode])
        
        base.accept("z", self.build_here,[Factory1])
        base.accept("x", self.build_here,[Factory2])
        base.accept("c", self.build_here,[Factory3])
        base.accept("v", self.build_here,[Factory4])
        base.accept("b", self.build_here,[Factory5])
        
        base.accept("h", self.build_here,[Reactor1])
        base.accept("j", self.build_here,[Reactor2])
        base.accept("m", self.build_here,[Mine])
        base.accept("p", self.build_here,[StarPort])
        
        base.accept("space", self.remove)
        base.accept("mouse1",self.build_click)
       
   
    def make_button(self,building_type,x,y):   
        def set():
            self.selected = building_type
            self.last = None
            self.help_text['text'] = building_type.help()
        b = DirectButton(image=loader.loadTexture("data/buildpics/"+building_type.__name__.lower()+".png"),
                         pos=(1.1-x*.12,1,.8-y*.12),
                         scale=.05,
                         command=set)
        b.reparentTo(self.button_node)
    
    def init_lines(self):
        lines = LineSegs()
        geom = lines.create()
        self.line_node = render.attachNewNode(geom)
        self.last = None
        self.lines = []
    
    def gen_lines(self):
        lines = LineSegs()
        lines.setThickness(6)
        lines.setColor(1,1,1)
        self.rs.gen_lines(lines)
        lines.setColor(0,0,1)
        self.ws.gen_lines(lines)
        lines.setColor(1,1,0)
        self.ps.gen_lines(lines)
        self.line_node.removeNode()
        geom = lines.create()
        self.line_node = render.attachNewNode(geom)   
        
    def lay(self,p,type,system):

        
        if self.last:
            self.last = game.bs.lay(self.last,p,type,system)
            if self.last :
                self.last = self.last.node.getPos()
                self.gen_lines()
        else:
            for r in system.roads:
                if (r.node.getPos()-p).length() < 1:
                    self.last = r.node.getPos()
            if not self.last: 
                self.last = p
                
        if self.last:
            self.ring.show()
            self.ring.setPos(self.last)
        else:
            self.ring.hide()
           
    def remove(self):
        if self.get_point():
            game.bs.remove(self.position)
         
    def build(self,p,type):
        if type == RoadNode: self.lay(p,type,game.rs)
        elif type == WaterNode: self.lay(p,type,game.ws)
        elif type == EnergyNode: self.lay(p,type,game.ps)
        else:
            building = game.bs.place(p,type)
            if building:
                building.setPos(p)  
                building.lookAt(game.rs.faceTo(p))
                building.setR(0)
                building.setP(0)
                
    def build_click(self):
        if self.selected:
            print "building",self.selected
            self.build_here(self.selected)

    def build_here(self,type):
        if self.get_point():
            self.build(self.position,type)
    
    def up(self):
        self.moveVec += Vec3(0,.1,0)
        
    def down(self):
        self.moveVec -= Vec3(0,.1,0)
        
    lastTime = 0
    def update(self,task):
        if time() - self.lastTime > 1:
            self.lastTime = time()
            try:
                game.bs.simulate()
                game.ms.update() 
                for pop,tree in progression:
                    if game.rv.population < pop:
                        if tree != self.build_tree:
                            self.build_tree = tree
                            self.button_node.removeNode()
                            self.button_node = NodePath("buttonNode")
                            self.button_node.reparentTo(aspect2d)
                            for y,branch in enumerate(self.build_tree):
                                for x,building_type in enumerate(branch):
                                    self.make_button(building_type,len(branch)-x,y)
                            break
                        else:
                            break
                self.info['text'] = self.rv.update()
            except:
                print_exc()
        return task.cont
    
    def mouse(self,task):
        
        md = base.win.getPointer(0)
        x = md.getX()
        y = md.getY()
        if self.rotating:#base.win.movePointer(0, base.win.getXSize()/2, base.win.getYSize()/2):
            self.heading += (x - self.oldXY[0]) * -0.2
            self.pitch += (y - self.oldXY[1]) * -0.2
            self.pitch = min(max( self.pitch, -80.0), 0.0 )
            base.zoomCam.setHpr( self.heading, self.pitch, 0 )
        self.oldXY = x,y
        
        if self.zooming > 0 :
            base.camera.setY(base.camera.getY() * 1.1)
            self.zooming = 0
            
        if self.zooming < 0 :
            base.camera.setY(base.camera.getY() * .9 )
            self.zooming = 0
            
        if base.camera.getY() > -1 :
            base.camera.setY(-1)
            
        if base.camera.getY() < -100 :
            base.camera.setY(-100)
        return task.cont

    def jump_cam(self):
        self.rotating = True
        if self.get_point():
            base.zoomCam.setPos(self.position)
        
    def init_picking(self):
        self.picker = CollisionTraverser()          
        self.queue  = CollisionHandlerQueue()       
        self.pickerNode = CollisionNode('mouseRay')
        self.pickerNP = base.camera.attachNewNode(self.pickerNode)
        self.pickerNode.setFromCollideMask(BitMask32.bit(1))
        self.pickerRay = CollisionRay()             
        self.pickerNode.addSolid(self.pickerRay)    
        self.picker.addCollider(self.pickerNP, self.queue)
        self.pickerNP.show()
        
    def init_lights(self):
        dlight = DirectionalLight('dlight')
        dlight.setColor(VBase4(1, 1, 1, 1))
        dlnp = render.attachNewNode(dlight)
        dlnp.setHpr(45, -60, 30)
        render.setLight(dlnp)
        
    def get_point(self):
        self.position = False
        if base.mouseWatcherNode.hasMouse():
            mousepos = base.mouseWatcherNode.getMouse()
            self.startY  = mousepos.getY()
            self.pickerRay.setFromLens(base.camNode, mousepos.getX(),mousepos.getY())
            nearPoint = render.getRelativePoint(camera, self.pickerRay.getOrigin())
            nearVec = render.getRelativeVector(camera, self.pickerRay.getDirection())
            self.picker.traverse(render)
            self.picker.showCollisions(render)
            if self.queue.getNumEntries() > 0:
                self.queue.sortEntries()
                self.position = self.queue.getEntry(0).getSurfacePoint(self.map)
                self.normal = self.queue.getEntry(0).getSurfaceNormal(self.map)
                self.normal.normalize()
                self.position = render.getRelativePoint(self.map, self.position)
        return self.position

 

def main(): 
    #render2d.hide()
    
    base.disableMouse()
    Game()
    base.zoomCam.setPos(50,50,10)
    base.zoomCam.setHpr(0,-45,0)
    base.camera.setPos(0,-100,0)
    run()